-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.

with Ada.Text_Io;

package body Warborg.Database is
   
   
   function "=" (Left, Right : in Wopr_Type) return Boolean is
   begin
      return left.Logname = Right.Logname;
   end "=";
   
   
   
   function "<" (Left, Right : in Missile_Type) return Boolean is            
   begin
      return False;
   end "<";
      
   package Sorting_Vector is new Weapon_Manager.Generic_Sorting("<");                        
   
   use  Weapon_Manager;
   use Nation_Manager;
   
   
   use Wopr_Manager;
   
   protected body Sgbd_Type is


      procedure Get_Wopr(Logname : in Logname_Type;
			 Wopr : in out Wopr_Type;
			 Success : in out boolean) is
	 
      begin
         
	 Text_Io.Put_Line("Getting wopr for " & Logname);
	 if not Wopr_Manager.Is_Empty(Woprs) then
	    for I in 1..Wopr_Manager.Last_Index(Woprs) loop
	       declare
		  Local : Wopr_Type := Wopr_Manager.Element(Woprs, I);
	       begin
		  Text_Io.Put_Line("Check wopr for " & local.Logname);
		  if local.logname = logname then
		     Wopr := Local;
		     Success := True;
		     exit;
		  end if;
	       end;     
	    end loop;
	 end if;
      exception
	 when others =>
	    Text_Io.Put_Line("                             exception in Get_Wopr in database.");
      end Get_Wopr;

      procedure Add_Wopr(Logname : in Logname_type;
			 Wopr : in Wopr_Type;
			 Success : out boolean) is
	 The_Worp : Wopr_Type := Wopr;
      begin	          
	 The_Worp.Logname := Logname;
	 Wopr_Manager.Insert(Woprs, Last_Index(Woprs) +1, The_Worp); 
	 Success := True;
      end Add_Wopr;
      
      
      procedure Get_Enemy(Logname : in Logname_Type;
                      Missiles : out Weapon_Manager.Vector) is
	 
	 
	 Temp : Weapon_Manager.Vector := Weapon_Manager.Empty_Vector;
	   
      begin
	 Missiles := Weapon_Manager.Empty_Vector;
	 for I in 1..Last_Index(Woprs) loop
	    if Element(Woprs, I).Logname /= Logname then
	       Insert(Temp, Last_Index(Temp) + 1, Element(Woprs, I).Weapons);
	    end if;
	 end loop;
	 Sorting_Vector.Sort(temp);
	 if not Is_Empty(Temp) then
	    for I in 1..Weapon_Manager.Extended_Index'Min(40000, Last_Index(Temp)) loop
	       Insert(Missiles, Last_Index(Missiles)+1, Element(Temp, I));
	    end loop;
	 end if;
	 
      end Get_Enemy;

      procedure Add_Missile(Logname : in Logname_Type;
                        Missile : in Missile_Type;
                        Success : out Boolean) is
	 Wopr : Wopr_Type;
      begin
	 
         Success := False;	 
	 for I in 1..Last_Index(Woprs) loop
	    if Element(Woprs, I).Logname = Logname then
	       Wopr := Element(Woprs, I);
	       Delete(Woprs, I);
	       if Wopr.Charge - Missile.Charge > 0.0 then
		  insert(Wopr.Weapons, Last_Index(Wopr.Weapons)+ 1, Missile);
		  Wopr.Charge := Wopr.Charge - Missile.Charge;
	       end if;
	       Wopr_Manager.Insert(Woprs, Last_Index(Woprs) +1, Wopr); 
	       Success := True;
	       exit;
	    end if;
	 end loop;
	 --Text_Io.Put_line("Add missile..");
      end Add_Missile;

      
      procedure Launch_All(Logname : in Logname_type) is

	 
	 
      begin
         --Text_Io.Put_line("Begin Launch all");
	 if not Wopr_Manager.Is_Empty(Woprs) then
	    for Wopr_Index in 1..Last_Index(Woprs) loop
	       
	       declare		  		  
		  Wopr : Wopr_Type := Element(Woprs, Wopr_Index);
		  Earned_Missiles : Weapon_Manager.Vector := Weapon_Manager.Empty_Vector;	 
	       begin
	       
		  if Wopr.Logname = Logname then
		     Delete(Woprs, Wopr_Index);
		  
		     if not Weapon_Manager.Is_Empty(Wopr.Weapons) then

		    Main :
			for Missile_index in 1..Weapon_Manager.Last_Index(Wopr.Weapons) loop
			   
			   
			   declare
			      Launched_Missile : constant Missile_type :=  Element(Wopr.Weapons, Missile_Index);						      
			   begin
			      --Text_Io.Put_line("Last index " & Integer'Image(Missile_Index));
			      --Text_Io.Put_line("Missile Target : ");
			      --Print(Launched_Missile.target);
			      
			      
			      
			      
			      
			      declare
				 Populations : Nation_Manager.Vector := Nation_Manager.Empty_Vector;		
			      begin			
				 --Text_Io.Put_line("Begin Launch all for local population");
				 Insert(Populations, 1, Wopr.Populations);		
				 Nation_Manager.Clear(Wopr.Populations);
				 for Population_index in 1..Nation_Manager.Last_Index(Populations) loop
				    declare
				       
				       Gps_Coordonates : constant Gps_Coordonates_Type := Nation_Manager.Element(Populations, Population_Index).Gps_Coordonates;
				       
				    begin
				       
				       if not Is_In(Launched_Missile.target, Gps_Coordonates, Float(Launched_Missile.Charge * Charge_Factor)) then
					  Insert(Wopr.Populations, 1, Nation_Manager.Element(Populations, Population_index));
				       end if;				 
				    end;
				 end loop;									
			      end;
			      
			      --Text_Io.Put_line("Population 2");
			      
			      
			      
			      declare
				 
				 
				 Winners : Wopr_Manager.Vector := Wopr_Manager.Empty_Vector;

			      begin
				 for Local_Wopr_Index in 1..Last_Index(Woprs) loop				 
				    declare
				       Temp : Wopr_Type := Element(Woprs, Local_Wopr_Index);
				    begin				       
				       --Text_Io.Put_line("Begin Launch all for others Woprs");			   
				       
				       declare
					  Populations : Nation_Manager.Vector := Nation_Manager.Empty_Vector;		
				       begin
					  --Text_Io.Put_line("Begin Launch all for others population");
					  Populations := Populations & Temp.Populations;
					  Nation_Manager.Clear(temp.Populations);
					  for Population_Index in 1..Nation_Manager.Last_Index(Populations) loop
					     declare
						Gps_Coordonates : constant Gps_Coordonates_Type := Nation_Manager.Element(Populations, Population_Index).Gps_Coordonates;
						--Map_Coordonates : Map_Coordonates_Type := To_Map_Coordonates(GPS_Coordonates);
					     begin
						
						if not Is_In(Launched_Missile.target, Gps_Coordonates, Float(Launched_Missile.Charge * Charge_Factor)) then
						   Insert(Temp.Populations, 1, Nation_Manager.Element(Populations, Population_index));
						end if;				    
					     end;				 
					  end loop;	
					  
					  
				       end;
				       
				       
				       
				       if not Is_Empty(Temp.Weapons) then
					  --Text_Io.Put_Line("Weapons index_last : " & Natural'Image(Weapon_Manager.Last_Index(Temp.Wopr.weapons)));
					  declare
					     enemy : weapon_Manager.Vector := Weapon_Manager.Empty_Vector;
					  begin					     					     
					     --Text_Io.Put_line("Begin Launch all for others weapons");				       
					     for Local_Weapon_Index in 1..Weapon_Manager.Last_Index(Temp.weapons) loop
						declare
						   Gps_Coordonates : constant Gps_Coordonates_Type := Weapon_Manager.Element(Temp.weapons, Local_Weapon_Index).Position;
						begin
						   --Text_Io.Put_Line("Enemi missile index : " & Natural'Image(Local_Weapon_index));
						   if Is_In(Launched_Missile.target, Gps_Coordonates, Float(Launched_Missile.Charge * Charge_Factor)) then
						      Text_Io.Put_line("Others missile destroyed");					  
						      Text_Io.Put_line("Missile position : ");
						      Print(Gps_coordonates);
						      

						      Wopr.Destroyed_Missiles := Wopr.Destroyed_Missiles + 1;
						      Text_Io.Put_line("Total Missile  : " & Natural'Image(Wopr.Destroyed_Missiles));
						   else
						      --Text_Io.Put_line("Save missile ");
						      Insert(enemy, Last_Index(enemy) +1, Weapon_Manager.Element(Temp.weapons, Local_Weapon_Index));
						   end if;			
						   
						exception
						   when others =>
						      Text_Io.Put_line("Exception before N-5");
						      raise;
						end;				 
					     end loop;	
					     
					     Weapon_Manager.Clear(Temp.Weapons);
					     Weapon_Manager.Insert(Temp.Weapons, 1, Enemy);
					     
					     
					  exception
					     when others =>
						Text_Io.Put_line("Exception before N-4");
						raise;
					     
					  end;
				       end if;

				       
				       
				       --Text_Io.Put_line("Check all for others Woprs");
				       if Is_In(Launched_Missile.target, Temp.Gps_coordonates, Float(Launched_Missile.Charge * Charge_Factor)) then
					  Text_Io.Put_line("                                                                Others Wopr destroyed");
					  --Text_Io.Put_line("Wopr coordonates");
					  --Print(Temp.Gps_Coordonates);
					  
					  --Text_Io.Put_line("Target coordonates");
					  --Print(Launched_Missile.target);

					  
					  
					  Wopr.Destroyed_Wopr := Wopr.Destroyed_Wopr + 1;
					  
					  if not Is_Empty(Temp.Weapons) then					     
					     Insert(Earned_Missiles, 1, Temp.Weapons);
					  end if;
					  
					  Text_Io.Put_line("Others Wopr destroyed done.");

				       else
					  Wopr_Manager.Insert(Winners, Last_Index(Winners) +1, Temp); 

				       end if;
				       
				       
				       
				    exception
				       when others =>
					  Text_Io.Put_line("Exception before N-3");
					  
					  raise;
					  
				    end;
				 end loop;
				 
				 Wopr_Manager.Clear(Woprs);
				 Woprs := Woprs & Winners;
				 Wopr_Manager.Clear(winners);
				 
			      exception
				 when others =>
				    Text_Io.Put_line("Exception before N-2");
				    
				    raise;
			      end;
			      
			      
			   exception
			      when others =>
				 Text_Io.Put_line("Exception before N-1");
				 
				 raise;
			   end;
			end loop main;
			
			
			Weapon_Manager.Clear(Wopr.Weapons);
			Weapon_Manager.Insert(Wopr.Weapons, 1, Earned_Missiles);		     
			Weapon_Manager.Clear(Earned_Missiles);
			Wopr.Earned_Missiles :=  Wopr.Earned_Missiles + Last_Index(Wopr.Weapons);

			
			
			
		     end if;		  		  	       
		     
		     Wopr.Charge := Wopr.Charge + Initial_Charge;
		     Wopr_Manager.Insert(Woprs, Last_Index(Woprs) +1, Wopr); 		     		     
		     
		     exit;
		     
		     
		  end if;            
		  
		  
	       exception
		  when others =>
		     Text_Io.Put_line("                                   exception in database : launch all failled.");
		     Wopr_Manager.Insert(Woprs, Last_Index(Woprs) +1, Wopr); 		     		     
		     raise;
	       end;
		  
	       
	       
	    end loop;
	    
	    
	 end if;
	 
	 Text_Io.Put_line("End Launch all");
      end Launch_All;
   end Sgbd_Type;
   
   
end Warborg.Database;
