
with Ada.Calendar;
use Ada;
with Gnat.SHA1;
with Ada.Containers.Vectors;

package Warborg is
   
   -- GPS define --
   
   subtype Longitude_Degree_Type is Float range -180.0..180.0;
   subtype Longitude_Minute_Type is Float range -60.0..60.0;
   subtype Longitude_Second_Type is Float range -60.0..60.0;
   
   type Longitude_Coordonate_Type is
      record
	 Degr_Lon : Longitude_Degree_Type := 0.0;
	 Minu_Lon : Longitude_Minute_Type := 0.0;
	 Seco_Lon : Longitude_Second_Type := 0.0;
      end record;
   
   subtype Latitude_Degree_Type is Float range -90.0..90.0;
   subtype Latitude_Minute_Type is Float range -60.0..60.0;
   subtype Latitude_Second_Type is Float range -60.0..60.0;   
   
   type Latitude_Coordonate_Type is
      record
	 Degr_Lat : Latitude_Degree_Type := 0.0;
	 Minu_Lat : Latitude_Minute_Type := 0.0;
	 Seco_Lat : Latitude_Second_Type := 0.0;
      end record;
   
   type GPS_Coordonates_Type is
      record
	 Longitude : Longitude_Coordonate_Type;
	 Latitude  : Latitude_Coordonate_Type;
      end record;
   
   subtype Decimal_Longitude_Type is Float range -180.0..180.0;
   subtype Decimal_Latitude_Type is Float range -90.0..90.0;   
   
   
   function To_GPS(Decimal_longitude : in Decimal_Longitude_Type;
		   Decimal_Latitude  : in Decimal_Latitude_Type)  return GPS_Coordonates_Type;
   
   procedure Print (GPS_Coordonates : GPS_Coordonates_Type);
   
   function latitude (Y : in Latitude_Coordonate_Type) return Decimal_Latitude_Type;
   function Longitude (X : in Longitude_Coordonate_Type) return Decimal_Longitude_Type;
   
   function Is_in(Target : in Gps_Coordonates_Type; item : in Gps_Coordonates_Type; Distance : in Float) return Boolean;
   
   type Side_Type is (Est, Ouest);
      
   function Side_Of(Position : in GPS_Coordonates_Type) return Side_Type;
   
   function "=" (Left, Right : in GPS_Coordonates_Type) return Boolean;
   
   
      -- Nation define --
   subtype Nation_Name_Type is String (1..200);
   type Nation_Type is tagged
      record
	 Name            : Nation_Name_Type := (others => Character'Val(32));
	 GPS_Coordonates : GPS_Coordonates_Type;
	 Population : Long_Long_Integer := 0;
      end record;      
   
   function "=" (Left, Right : in Nation_Type) return Boolean;
   
   type Nation_Index_Type is new Long_Integer range 1..Long_integer'Last;
   
   package Nation_Manager is new Ada.Containers.Vectors(Nation_Index_Type, Nation_Type, "=");
   
   -- Weapon define --
   
   subtype Charge_Type is Long_Float range 0.0..Long_Float'Last;
   
   type Missile_Type is
      record
	 Armed    : Boolean := True;
	 Charge   : Charge_type := 1.0;
	 Position : Gps_Coordonates_Type;
	 Target   : Gps_Coordonates_Type;
      end record;
	    
   function "=" (Left, Right : in Missile_Type) return Boolean;
   
   subtype Missile_Index_Type is Positive range 1..1_000_000;
   
   package Weapon_Manager is new Ada.Containers.Vectors(Missile_Index_Type, Missile_Type, "=");
   
   Charge_Factor : constant Long_Float := 1.0/60.0;
   
   Initial_Charge : Charge_Type := 900.0;
   
   -- User define --
   
   subtype Logname_Type is String (1..32);
   
   use Gnat.SHA1;
   
   type User_Type is tagged
      record
	 Logname : Logname_Type := (others => Character'Val(32));
	 Passwd : Message_Digest;
      end record;         
   
      procedure Initialize(User : out User_Type;
			Name : in Logname_Type);
   -- Get player named "Name".
   

   
   procedure Set_Pass(User : in out User_Type;
		      Pass : in Message_Digest);
   -- Set password for player.
   
   function Check_Pass(User : in User_Type;
		       Pass : in Message_Digest) return Boolean;
   -- Return True is getted password is valid. False else.      

   
   -- WOPR define --
   
   type Lertcon_Type is new Positive range 1..5;
   
   type Wopr_Type is
      record
	 Logname            : Logname_Type := (others => Character'Val(32));
	 GPS_Coordonates    : GPS_Coordonates_Type := ((0.0, 0.0, 0.0), (0.0, 0.0, 0.0));
	 Charge             : Charge_Type := Charge_Type'First;
	 Start_Time         : Calendar.Time := Calendar.Clock;	 
	 Stats_Time         : Calendar.Time := Calendar.Clock;	 	 	 
	 	 
	 Destroyed_Wopr     : Natural := 0;
         Earned_Missiles    : Natural := 0;
	 Destroyed_Missiles : Natural := 0;	 
	 	 
	 Side               : Side_Type := Ouest;
	 
	 Populations        : Nation_Manager.Vector := Nation_Manager.Empty_Vector; 
	 Weapons            : Weapon_Manager.Vector := Weapon_Manager.Empty_Vector;
	 Enemy              : Weapon_Manager.Vector := Weapon_Manager.Empty_Vector;
	 Defcon             : Lertcon_Type := 5;
	 
	 Self_Autorized     : Boolean := True;
      end record;
   
   
   type Command_Type is
     
     (
      Login,
      Wopr_Create,
      Wopr_Get,
      Missile_Create,
      Launch_All,
      Enemy_Get,
      Logout
     );

   
end Warborg;
